@@ -755,6 +755,8 @@ class TestYara(unittest.TestCase):
         r = yara.compile(p2)
         self.assertTrue(len(r.match(data='dummy')) == 2)
 
+        self.assertRaises(yara.SyntaxError, yara.compile, source='include "test"', includes=False)
+
     def testExternals(self):
 
         r = yara.compile(source='rule test { condition: ext_int == 15 }', externals={'ext_int': 15})
@@ -827,6 +829,23 @@ class TestYara(unittest.TestCase):
         self.assertTrue(rule_data['matches'])
         self.assertTrue(rule_data['rule'] == 'test')
 
+        rule_data = None
+
+        r = yara.compile(source='rule test { condition: false }')
+        r.match(data='dummy', callback=callback, which_callbacks=yara.CALLBACK_NON_MATCHES)
+
+        self.assertTrue(rule_data['rule'] == 'test')
+
+    def testIncludeCallback(self):
+
+        def callback(requested_filename, filename, namespace):
+            if requested_filename == 'foo':
+                return 'rule included {condition: true }'
+            return None
+
+        r = yara.compile(source='include "foo" rule r { condition: included }', include_callback=callback)
+        self.assertTrue(r.match(data='dummy'))
+
     def testCompare(self):
 
         r = yara.compile(sources={
@@ -939,6 +958,25 @@ class TestYara(unittest.TestCase):
         self.assertTrue(data['constants']['one'] == 1)
         self.assertTrue(data['constants']['two'] == 2)
 
+    def testRulesIterator(self):
+
+        rules = yara.compile(
+            source='''
+            rule test1 { condition: false }
+            rule test2 { condition: false }
+            rule test3 { condition: false }
+            ''')
+
+        for i, r in enumerate(rules, start=1):
+           self.assertTrue(r.identifier == 'test%d' % i)
+
+        it = iter(rules)
+        r = next(it)
+        self.assertTrue(r.identifier == 'test1')
+        r = next(it)
+        self.assertTrue(r.identifier == 'test2')
+        r = next(it)
+        self.assertTrue(r.identifier == 'test3')
 
 
 if __name__ == "__main__":
