@@ -68,6 +68,10 @@ This module allows you to apply YARA rules to files or strings.\n\
 For complete documentation please visit:\n\
 https://plusvic.github.io/yara\n"
 
+#if defined(_WIN32) || defined(__CYGWIN__)
+#include <string.h>
+#define strdup _strdup
+#endif
 
 // Match object
 
@@ -389,6 +393,7 @@ typedef struct _CALLBACK_DATA
   PyObject* callback;
   PyObject* modules_data;
   PyObject* modules_callback;
+  int which;
 
 } CALLBACK_DATA;
 
@@ -424,7 +429,7 @@ PyObject* convert_object_to_python(
     case OBJECT_TYPE_STRING:
       if (object->value.ss != NULL)
         result = PyBytes_FromStringAndSize(
-            object->value.ss->c_string, 
+            object->value.ss->c_string,
             object->value.ss->length);
       break;
 
@@ -551,6 +556,10 @@ PyObject* convert_dictionary_to_python(
 }
 
 
+#define CALLBACK_MATCHES 0x01
+#define CALLBACK_NON_MATCHES 0x02
+#define CALLBACK_ALL CALLBACK_MATCHES | CALLBACK_NON_MATCHES
+
 int yara_callback(
     int message,
     void* message_data,
@@ -578,6 +587,7 @@ int yara_callback(
   PyObject* module_data;
   PyObject* callback_result;
   PyObject* module_info_dict;
+  int which = ((CALLBACK_DATA*) user_data)->which;
 
   Py_ssize_t data_size;
   PyGILState_STATE gil_state;
@@ -587,7 +597,8 @@ int yara_callback(
   if (message == CALLBACK_MSG_SCAN_FINISHED)
     return CALLBACK_CONTINUE;
 
-  if (message == CALLBACK_MSG_RULE_NOT_MATCHING && callback == NULL)
+  if (message == CALLBACK_MSG_RULE_NOT_MATCHING &&
+      (callback == NULL || (which & CALLBACK_MATCHES)))
     return CALLBACK_CONTINUE;
 
   if (message == CALLBACK_MSG_IMPORT_MODULE && modules_data == NULL)
@@ -758,7 +769,9 @@ int yara_callback(
     }
   }
 
-  if (callback != NULL)
+  if (callback != NULL &&
+      ((message == CALLBACK_MSG_RULE_MATCHING && (which & CALLBACK_MATCHES)) ||
+       (message == CALLBACK_MSG_RULE_NOT_MATCHING && (which & CALLBACK_NON_MATCHES))))
   {
     Py_INCREF(callback);
 
@@ -903,8 +916,8 @@ PyObject* handle_error(
   {
     case ERROR_COULD_NOT_ATTACH_TO_PROCESS:
       return PyErr_Format(
-        YaraError,
-        "access denied");
+          YaraError,
+          "access denied");
     case ERROR_INSUFFICIENT_MEMORY:
       return PyErr_NoMemory();
     case ERROR_COULD_NOT_OPEN_FILE:
@@ -936,6 +949,11 @@ PyObject* handle_error(
           YaraError,
           "external variable \"%s\" was already defined with a different type",
           extra);
+    case ERROR_UNSUPPORTED_FILE_VERSION:
+      return PyErr_Format(
+          YaraError,
+          "rules file \"%s\" is incompatible with this version of YARA",
+          extra);
     default:
       return PyErr_Format(
           YaraError,
@@ -1273,6 +1291,7 @@ static PyObject* Rules_next(
 
   if (RULE_IS_NULL(rules->iter_current_rule))
   {
+    rules->iter_current_rule = rules->rules->rules_list_head;
     PyErr_SetNone(PyExc_StopIteration);
     return NULL;
   }
@@ -1325,7 +1344,7 @@ static PyObject* Rules_match(
   static char* kwlist[] = {
       "filepath", "pid", "data", "externals",
       "callback", "fast", "timeout", "modules_data",
-      "modules_callback", NULL
+      "modules_callback", "which_callbacks", NULL
       };
 
   char* filepath = NULL;
@@ -1348,11 +1367,12 @@ static PyObject* Rules_match(
   callback_data.callback = NULL;
   callback_data.modules_data = NULL;
   callback_data.modules_callback = NULL;
+  callback_data.which = CALLBACK_ALL;
 
   if (PyArg_ParseTupleAndKeywords(
         args,
         keywords,
-        "|sis#OOOiOO",
+        "|sis#OOOiOOi",
         kwlist,
         &filepath,
         &pid,
@@ -1363,7 +1383,8 @@ static PyObject* Rules_match(
         &fast,
         &timeout,
         &callback_data.modules_data,
-        &callback_data.modules_callback))
+        &callback_data.modules_callback,
+        &callback_data.which))
   {
     if (filepath == NULL && data == NULL && pid == 0)
     {
@@ -1694,6 +1715,103 @@ void raise_exception_on_error_or_warning(
 
 ////////////////////////////////////////////////////////////////////////////////
 
+const char* yara_include_callback(
+    const char* include_name,
+    const char* calling_rule_filename,
+    const char* calling_rule_namespace,
+    void* user_data)
+{
+  PyObject* result;
+  PyObject* callback = (PyObject*) user_data;
+  PyObject* py_incl_name = NULL;
+  PyObject* py_calling_fn = NULL;
+  PyObject* py_calling_ns = NULL;
+  PyObject* type = NULL;
+  PyObject* value = NULL;
+  PyObject* traceback = NULL;
+
+  const char* cstring_result = NULL;
+
+  PyGILState_STATE gil_state = PyGILState_Ensure();
+
+  if (include_name != NULL)
+  {
+    py_incl_name = PY_STRING(include_name);
+  }
+  else //safeguard: should never happen for 'include_name'
+  {
+    py_incl_name = Py_None;
+    Py_INCREF(py_incl_name);
+  }
+
+  if (calling_rule_filename != NULL)
+  {
+    py_calling_fn = PY_STRING(calling_rule_filename);
+  }
+  else
+  {
+    py_calling_fn = Py_None;
+    Py_INCREF(py_calling_fn);
+  }
+
+  if (calling_rule_namespace != NULL)
+  {
+    py_calling_ns = PY_STRING(calling_rule_namespace);
+  }
+  else
+  {
+    py_calling_ns = Py_None;
+    Py_INCREF(py_calling_ns);
+  }
+
+  PyErr_Fetch(&type, &value, &traceback);
+
+  result = PyObject_CallFunctionObjArgs(
+      callback,
+      py_incl_name,
+      py_calling_fn,
+      py_calling_ns,
+      NULL);
+
+  PyErr_Restore(type, value, traceback);
+
+  Py_DECREF(py_incl_name);
+  Py_DECREF(py_calling_fn);
+  Py_DECREF(py_calling_ns);
+
+  if (result != NULL && result != Py_None && PY_STRING_CHECK(result))
+  {
+    //transferring string ownership to C code
+    cstring_result = strdup(PY_STRING_TO_C(result));
+  }
+  else
+  {
+    if (PyErr_Occurred() == NULL)
+    {
+      PyErr_Format(PyExc_TypeError,
+          "'include_callback' function must return a yara rules as an ascii "
+          "or unicode string");
+    }
+  }
+
+  Py_XDECREF(result);
+  PyGILState_Release(gil_state);
+
+  return cstring_result;
+}
+
+void yara_include_free(
+    const char* result_ptr,
+    void* user_data)
+{
+  if (result_ptr != NULL)
+  {
+    free((void*) result_ptr);
+  }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
 static PyObject* yara_compile(
     PyObject* self,
     PyObject* args,
@@ -1701,7 +1819,7 @@ static PyObject* yara_compile(
 {
   static char *kwlist[] = {
     "filepath", "source", "file", "filepaths", "sources",
-    "includes", "externals", "error_on_warning", NULL};
+    "includes", "externals", "error_on_warning", "include_callback", NULL};
 
   YR_COMPILER* compiler;
   YR_RULES* yara_rules;
@@ -1718,6 +1836,7 @@ static PyObject* yara_compile(
   PyObject* includes = NULL;
   PyObject* externals = NULL;
   PyObject* error_on_warning = NULL;
+  PyObject* include_callback = NULL;
 
   Py_ssize_t pos = 0;
 
@@ -1731,7 +1850,7 @@ static PyObject* yara_compile(
   if (PyArg_ParseTupleAndKeywords(
         args,
         keywords,
-        "|ssOOOOOO",
+        "|ssOOOOOOO",
         kwlist,
         &filepath,
         &source,
@@ -1740,8 +1859,10 @@ static PyObject* yara_compile(
         &sources_dict,
         &includes,
         &externals,
-        &error_on_warning))
+        &error_on_warning,
+        &include_callback))
   {
+
     error = yr_compiler_create(&compiler);
 
     if (error != ERROR_SUCCESS)
@@ -1775,7 +1896,8 @@ static PyObject* yara_compile(
       if (PyBool_Check(includes))
       {
         // PyObject_IsTrue can return -1 in case of error
-        compiler->allow_includes = (PyObject_IsTrue(includes) == 1);
+        if (PyObject_IsTrue(includes) == 1)
+          yr_compiler_set_include_callback(compiler, NULL, NULL, NULL);
       }
       else
       {
@@ -1786,6 +1908,23 @@ static PyObject* yara_compile(
       }
     }
 
+    if (include_callback != NULL)
+    {
+      if (!PyCallable_Check(include_callback))
+      {
+        yr_compiler_destroy(compiler);
+        return PyErr_Format(
+            PyExc_TypeError,
+            "'include_callback' must be callable");
+      }
+
+      yr_compiler_set_include_callback(
+          compiler,
+          yara_include_callback,
+          yara_include_free,
+          include_callback);
+    }
+
     if (externals != NULL && externals != Py_None)
     {
       if (PyDict_Check(externals))
@@ -1805,6 +1944,8 @@ static PyObject* yara_compile(
       }
     }
 
+    Py_XINCREF(include_callback);
+
     if (filepath != NULL)
     {
       fh = fopen(filepath, "r");
@@ -1826,12 +1967,15 @@ static PyObject* yara_compile(
     else if (file != NULL)
     {
       fd = dup(PyObject_AsFileDescriptor(file));
-      if (fd != -1) {
+
+      if (fd != -1)
+      {
         fh = fdopen(fd, "r");
         error = yr_compiler_add_file(compiler, fh, NULL, NULL);
         fclose(fh);
       }
-      else {
+      else
+      {
         result = PyErr_Format(
             PyExc_TypeError,
             "'file' is not a file object");
@@ -1954,6 +2098,7 @@ static PyObject* yara_compile(
     }
 
     yr_compiler_destroy(compiler);
+    Py_XDECREF(include_callback);
   }
 
   return result;
@@ -2129,6 +2274,9 @@ MOD_INIT(yara)
 
   PyModule_AddIntConstant(m, "CALLBACK_CONTINUE", 0);
   PyModule_AddIntConstant(m, "CALLBACK_ABORT", 1);
+  PyModule_AddIntConstant(m, "CALLBACK_MATCHES", CALLBACK_MATCHES);
+  PyModule_AddIntConstant(m, "CALLBACK_NON_MATCHES", CALLBACK_NON_MATCHES);
+  PyModule_AddIntConstant(m, "CALLBACK_ALL", CALLBACK_ALL);
   PyModule_AddStringConstant(m, "__version__", YR_VERSION);
   PyModule_AddStringConstant(m, "YARA_VERSION", YR_VERSION);
   PyModule_AddIntConstant(m, "YARA_VERSION_HEX", YR_VERSION_HEX);
